# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - Queue changes for review

    Queue changed pages edited by unprivileged users for review by the
    ApproveChanges action.

    @copyright: 2011 Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL, see COPYING for details.
"""

from MoinMoin.PageEditor import PageEditor
from MoinMoin.events import PagePreSaveEvent, Abort
import re

def match_queue_pages(pagename, queued_changes_area):

    "Return a filter function which matches subpages of 'pagename'."

    return re.compile(ur"^%s/%s/.*$" % (re.escape(pagename), re.escape(queued_changes_area)), re.UNICODE).match

def handle_presave(event):
    request = event.request
    _ = request.getText

    approved_editors_group = getattr(request.cfg, "approved_editors_group", "ApprovedGroup")
    queued_changes_area = getattr(request.cfg, "queued_changes_area", "ApprovalQueue")

    pagename = event.page_editor.page_name
    parts = pagename.split("/")

    # Saving into queues has to be permitted or the mechanism will keep trying
    # to save into a queue of the specified page.

    if len(parts) > 2 and parts[-2] == queued_changes_area:
        return None

    # For normal pages, the user has to be approved. Otherwise, the page will be
    # saved into a queue.

    elif not request.user.valid or (
        not request.dicts.has_member(approved_editors_group, request.user.name) and \
        not request.user.isSuperUser()):

        # Get a queue number by listing the subpages of the queue area and
        # finding the highest queue number, then adding one to it.

        queue_number = -1

        for name in request.rootpage.getPageList(exists=1, filter=match_queue_pages(pagename, queued_changes_area)):
            number = name.split("/")[-1]
            if number.isdigit():
                queue_number = max(queue_number, int(number))

        queue_number += 1

        # Save the page in the queue.

        new_page = PageEditor(request, "%s/%s/%s" % (pagename, queued_changes_area, queue_number))

        try:
            new_page.saveText(event.new_text, 0)
        except PageEditor.Unchanged:
            pass

        # Abort the page save operation.

        return Abort(_("Your changes have been queued for approval."))

def handle(event):
    if isinstance(event, PagePreSaveEvent):
        return handle_presave(event)

# vim: tabstop=4 expandtab shiftwidth=4
